@@ -71,6 +71,8 @@ private:
     void drawBarNumber();
     void updateMidiTask();
 
+    QString accuracyText;
+    int accuracyBarStart = 0;
 
     QColor m_backgroundColor;
     QtWindow* m_qtWindow;  // The parent Window
