@@ -26,11 +26,9 @@
 */
 /*********************************************************************************/
 
-
 #include "Notation.h"
 #include "Cfg.h"
 
-
 #define OPTION_DEBUG_NOTATION     0
 #if OPTION_DEBUG_NOTATION
 #define ppDEBUG_NOTATION(args)     ppLogDebug  args
@@ -38,7 +36,6 @@
 #define ppDEBUG_NOTATION(args)
 #endif
 
-
 #define MERGESLOT_NOTE_INDEX        0
 #define MERGESLOT_BEATMARK_INDEX    1
 
@@ -60,7 +57,7 @@ bool CSlot::addSymbol(CSymbol symbol)
                 return true;
             break;
         }
-        // move the previous entry up one possition
+        // move the previous entry up one position
         m_symbols[i+1] = m_symbols[i];
     }
     m_symbols[i+1] = symbol;
@@ -193,7 +190,7 @@ accidentalModifer_t CNotation::detectSuppressedNatural(int note)
     }
     if (pBackLink)
     {
-        pNoteState->setBackLink(0);
+        pNoteState->setBackLink(nullptr);
         pBackLink->setBarChange(-1); // this prevents further suppression on the original note
     }
 
@@ -212,7 +209,6 @@ void CNotation::setupNotationParamaters()
     cfg_param[NOTATE_semibreveBoundary]  = CMidiFile::ppqnAdjust(DEFAULT_PPQN*4 + 10);
 }
 
-
 void CNotation::calculateScoreNoteLength()
 {
     if (!Cfg::experimentalNoteLength)
