@@ -5,7 +5,7 @@
 
     @author         L. J. Barman
 
-    Copyright (c)   2008-2013, L. J. Barman, all rights reserved
+    Copyright (c)   2008-2020, L. J. Barman and others, all rights reserved
 
     This file is part of the PianoBooster application
 
@@ -47,8 +47,11 @@
 #include "GuiTopBar.h"
 #include "GuiSidePanel.h"
 #include "QtWindow.h"
-#include "version.txt"
+#include "version.h"
 
+#if WITH_INTERNAL_FLUIDSYNTH
+#include "MidiDeviceFluidSynth.h"
+#endif
 
 #define OPTION_DEBUG_SETTINGS   0
 #if OPTION_DEBUG_SETTINGS
@@ -57,7 +60,6 @@
 #define debugSettings(args)
 #endif
 
-
 CSettings::CSettings(QtWindow *mainWindow) : QSettings(CSettings::IniFormat, CSettings::UserScope, "PianoBooster", "Piano Booster"),
                                  m_mainWindow(mainWindow)
 {
@@ -71,15 +73,8 @@ CSettings::CSettings(QtWindow *mainWindow) : QSettings(CSettings::IniFormat, CSe
     CNotation::setCourtesyAccidentals(value("Score/CourtesyAccidentals", false ).toBool());
     m_followThroughErrorsEnabled = value("Score/FollowThroughErrors", false ).toBool();
 
-
     // load Fluid settings
-
-    QString soundFontNames_1 = value("FluidSynth/SoundFont2_1","").toString();
-    if (!soundFontNames_1.isEmpty()) addFluidSoundFontName(soundFontNames_1);
-
-    QString soundFontNames_2 = value("FluidSynth/SoundFont2_2","").toString();
-    if (!soundFontNames_2.isEmpty()) addFluidSoundFontName(soundFontNames_2);
-
+    setFluidSoundFontNames( value("FluidSynth/SoundFont").toStringList());
 }
 
 void CSettings::setDefaultValue(const QString & key, const QVariant & value )
@@ -90,7 +85,6 @@ void CSettings::setDefaultValue(const QString & key, const QVariant & value )
     setValue(key, value);
 }
 
-
 void CSettings::init(CSong* song, GuiSidePanel* sidePanel, GuiTopBar* topBar)
 {
     m_song = song;
@@ -98,7 +92,6 @@ void CSettings::init(CSong* song, GuiSidePanel* sidePanel, GuiTopBar* topBar)
     m_guiTopBar = topBar;
 }
 
-
 void CSettings::setNoteNamesEnabled(bool value) {
     m_noteNamesEnabled = value;
     setValue("Score/NoteNames", value );
@@ -115,7 +108,6 @@ void CSettings::setTutorPagesEnabled(bool value) {
     updateTutorPage();
 }
 
-
 void CSettings::setCourtesyAccidentals(bool value) {
     CNotation::setCourtesyAccidentals(value);
     setValue("Score/CourtesyAccidentals", value );
@@ -133,7 +125,7 @@ QDomElement CSettings::openDomElement(QDomElement parent, const QString & elemen
 
     debugSettings(("openDomElement1 %s %s %s", qPrintable(parent.tagName()), qPrintable(elementName), qPrintable(elementName)));
     // There should be only a single element without a name
-    // there should be lots of elemens but only one with this tag name
+    // there should be lots of elements but only one with this tag name
 
     QDomNode n = parent.firstChild();
     while(!n.isNull())
@@ -157,7 +149,7 @@ QDomElement CSettings::openDomElement(QDomElement parent, const QString & elemen
 
     if (wantedElement.isNull())
     {
-        // Create the element because it does not exsist
+        // Create the element because it does not exist
         wantedElement = m_domDocument.createElement(elementName);
         if (!attributeName.isEmpty() )
             wantedElement.setAttribute("name", attributeName);
@@ -186,7 +178,6 @@ void CSettings::loadSongSettings()
     m_guiSidePanel->setCurrentHand(m_domSong.attribute("hand", "both" ));
     m_guiTopBar->setSpeed(m_domSong.attribute("speed", "100" ).toInt());
 
-
     // -1 means none and -2 means not set
     int left = m_domSong.attribute("leftHandMidiChannel", "-2").toInt();
     int right = m_domSong.attribute("rightHandMidiChannel", "-2").toInt();
@@ -195,7 +186,6 @@ void CSettings::loadSongSettings()
     loadHandSettings();
 }
 
-
 void CSettings::saveSongSettings()
 {
     m_domSong.setAttribute("hand", partToHandString(m_song->getActiveHand()));
@@ -213,7 +203,6 @@ void CSettings::loadBookSettings()
         m_currentSongName = lastSong;
 }
 
-
 void CSettings::saveBookSettings()
 {
     if (!m_currentBookName.isEmpty())
@@ -224,7 +213,6 @@ void CSettings::saveBookSettings()
     saveSongSettings();
 }
 
-
 void CSettings::loadXmlFile()
 {
     m_domDocument.documentElement().clear();
@@ -286,34 +274,32 @@ void CSettings::updateTutorPage()
     QFileInfo fileInfo(getCurrentSongLongFileName());
     const char* EXTN = ".html";
 
-    QString fileBase = fileInfo.absolutePath() + "/InfoPages/" + fileInfo.completeBaseName() + "_";
 
-    QString locale = value("General/lang",QLocale::system().bcp47Name()).toString();
 
     if (m_tutorPagesEnabled)
     {
+        QString locale = selectedLangauge();
+
+        QString tutorFilePath = fileInfo.absolutePath() + "/InfoPages/" + locale + "/";
         QFileInfo tutorFile;
-        tutorFile.setFile(fileBase + locale + EXTN);
+        tutorFile.setFile(tutorFilePath + fileInfo.completeBaseName() + EXTN);
         if (tutorFile.exists())
         {
             m_mainWindow->loadTutorHtml(tutorFile.absoluteFilePath());
             return;
         }
-        int n = locale.indexOf("_");
 
-        if (n > 0)
+        tutorFilePath = fileInfo.absolutePath() + "/InfoPages/en/";
+        tutorFile.setFile(tutorFilePath + fileInfo.completeBaseName() + EXTN);
+
+        if (tutorFile.exists())
         {
-            locale = locale.left(n);
-            tutorFile.setFile(fileBase + locale + EXTN);
-            if (tutorFile.exists())
-            {
-                m_mainWindow->loadTutorHtml(tutorFile.absoluteFilePath());
-                return;
-            }
+            m_mainWindow->loadTutorHtml(tutorFile.absoluteFilePath());
+            return;
         }
 
-        locale = "en";
-        tutorFile.setFile(fileBase + locale + EXTN);
+        // The previous location of the tutor pages English only
+        tutorFile.setFile(fileInfo.absolutePath() + "/InfoPages/" + fileInfo.completeBaseName() + "_en" + EXTN);
         if (tutorFile.exists())
         {
             m_mainWindow->loadTutorHtml(tutorFile.absoluteFilePath());
@@ -321,7 +307,6 @@ void CSettings::updateTutorPage()
         }
     }
     m_mainWindow->loadTutorHtml(QString());
-
 }
 
 void CSettings::openSongFile(const QString & filename)
@@ -366,7 +351,6 @@ QStringList CSettings::getSongList()
     dirSongs.setFilter(QDir::Files);
     QStringList fileNames = dirSongs.entryList();
 
-
     QStringList songNames;
     for (int i = 0; i < fileNames.size(); i++)
     {
@@ -388,7 +372,6 @@ QStringList CSettings::getBookList()
     return dirBooks.entryList();
 }
 
-
 void CSettings::writeSettings()
 {
 
@@ -397,7 +380,6 @@ void CSettings::writeSettings()
     saveXmlFile();
 }
 
-
 void CSettings::loadSettings()
 {
     unzipBoosterMusicBooks();
@@ -418,6 +400,8 @@ void CSettings::loadSettings()
 //    if (!songName.isEmpty())
 //        openSongFile( songName );
 
+    setupDefaultSoundFont();
+
     updateWarningMessages();
     updateTutorPage();
 
@@ -428,33 +412,26 @@ void CSettings::unzipBoosterMusicBooks()
     // Set default value
     const QString ZIPFILENAME("BoosterMusicBooks.zip");
 
-
     if (value("PianoBooster/MusicRelease", 0).toInt() < MUSIC_RELEASE)
     {
-        QString resourceDir = QApplication::applicationDirPath() + "/../music/";
+        QString musicSrcDir = QApplication::applicationDirPath() + "/";
 
-        ppLogTrace("unzipBoosterMusicBooks resourceDir1 %s", qPrintable(resourceDir));
+        ppLogTrace("unzipBoosterMusicBooks resourceDir1 %s", qPrintable(musicSrcDir));
 
-        if (!QFile::exists(resourceDir + ZIPFILENAME))
-            resourceDir = QApplication::applicationDirPath() + "/../../music/";
-        ppLogTrace("unzipBoosterMusicBooks resourceDir2 %s", qPrintable(resourceDir));
-
-        if (!QFile::exists(resourceDir + ZIPFILENAME))
+        if (!QFile::exists(musicSrcDir + ZIPFILENAME))
         {
 #if defined (Q_OS_LINUX) || defined (Q_OS_UNIX)
-           resourceDir=QString(PREFIX)+"/"+QString(DATA_DIR)+"/music/";
+            musicSrcDir=Util::dataDir()+"/music/";
 #endif
 #ifdef Q_OS_DARWIN
-            resourceDir = QApplication::applicationDirPath() + "/../Resources/music/";
+            musicSrcDir = QApplication::applicationDirPath() + "/../Resources/music/";
 #endif
         }
 
         ppLogInfo(qPrintable("applicationDirPath=" + QApplication::applicationDirPath()));
-        ppLogTrace("resourceDir3 %s", qPrintable(resourceDir));
+        ppLogTrace("resourceDir %s", qPrintable(musicSrcDir));
 
-
-        QFileInfo zipFile(resourceDir +  ZIPFILENAME);
-        ppLogTrace("xx %s", qPrintable(zipFile.filePath()));
+        QFileInfo zipFile(musicSrcDir +  ZIPFILENAME);
 
         QDir destMusicDir;
 
@@ -476,15 +453,13 @@ void CSettings::unzipBoosterMusicBooks()
 
 #ifndef _WIN32
 
-        // on windows the the installer does the unzipping
+        // on windows the installer does the unzipping
         if (!zipFile.exists() )
         {
             ppLogError(qPrintable("Cannot find " + ZIPFILENAME));
             return;
         }
 
-
-
         QProcess unzip;
         unzip.start("unzip", QStringList() << "-o" << zipFile.filePath() << "-d" << destMusicDir.path() );
         ppLogInfo(qPrintable("running unzip -o " + zipFile.filePath() + " -d " + destMusicDir.path()) );
@@ -498,7 +473,6 @@ void CSettings::unzipBoosterMusicBooks()
                  // the line is available in buf
         }
 
-
         if (!unzip.waitForFinished())
         {
              ppLogError("unzip failed");
@@ -562,3 +536,59 @@ void CSettings::updateWarningMessages()
         m_warningMessage.clear();
 }
 
+void CSettings::setupDefaultSoundFont(){
+#if WITH_INTERNAL_FLUIDSYNTH
+
+    QFileInfo defaultSoundFont = QFileInfo();
+
+    if (getFluidSoundFontNames().empty() && !m_song->validMidiOutput() && !contains("LastSoundFontDir"))
+    {
+        QString appPath = qgetenv("APPIMAGE");
+        if (!appPath.isEmpty())
+        {
+             appPath= QFileInfo(appPath).path();
+        }
+        else {
+            appPath = QApplication::applicationDirPath();
+        }
+
+        QDir directory(appPath);
+        directory.cd("soundfont");
+        QStringList dirList = directory.entryList(QStringList(),QDir::AllEntries);
+        foreach(QString filename, dirList)
+        {
+            // Find the first sound fount file
+            if ( filename.endsWith(".sf2", Qt::CaseInsensitive ) )
+            {
+                defaultSoundFont.setFile(directory.path(), filename);
+                break;
+            }
+        }
+
+#if defined (Q_OS_LINUX) || defined (Q_OS_UNIX)
+        if (!defaultSoundFont.isFile())
+        {
+            QStringList possibleSoundFontFolders = {"/usr/share/soundfonts","/usr/share/sounds/sf2"};
+            for (QString soundFontFolder:possibleSoundFontFolders)
+            {
+                // fluid-soundfont-gm Fluid (R3) General MIDI SoundFont (GM) package
+                QFileInfo foundSoundFont(soundFontFolder, "FluidR3_GM.sf2");
+                if (foundSoundFont.isFile())
+                {
+                       defaultSoundFont=foundSoundFont;
+                       break;
+                }
+            }
+        }
+#endif
+
+        if (defaultSoundFont.isFile()) {
+            setFluidSoundFontNames(defaultSoundFont.filePath());
+            setValue("Midi/Output",CMidiDeviceFluidSynth::getFluidInternalName() );
+            setValue("LastSoundFontDir", defaultSoundFont.path());
+            saveSoundFontSettings();
+            m_song->openMidiPort(CMidiDevice::MIDI_OUTPUT, CMidiDeviceFluidSynth::getFluidInternalName());
+        }
+    }
+#endif
+}
